#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# Use this to get interfaces for a host
# Requires ControllerBase/Chained - must come after this in use base
# because we set the template before menus is actually called.
# This is because errors need to use the blank wrapper

package Opsview::Web::ControllerBase::Interfaces;
use base qw(Catalyst::Controller);

use NMIS;
use func;
use csv;

sub do_interfaces : Private {
    my ( $self, $c, $graphtype ) = @_;
    $graphtype = "bits" unless $graphtype =~ /^(bits|autil|pkts|epkts)$/;
    $c->stash( graphtype => $graphtype );
    $c->stash( template  => "interfaces" );
    delete $c->stash->{wrapper};
    my $host     = $c->stash->{object};
    my $node     = $host->ip;
    my $csv_file = "$NMIS::config{'<nmis_var>'}/$node-interface.dat";
    if ( !-e $csv_file ) {
        $c->stash( error => $c->loc( "No interfaces on [_1]", $host->name ) );
        $c->detach("/soft_error");
    }
    my %interfaceTable = loadCSV( $csv_file, "ifIndex", "\t" );

    # Check again because could be error loading file
    unless (%interfaceTable) {
        $c->stash( error => $c->loc( "No interfaces on [_1]", $host->name ) );
        $c->detach("/soft_error");
    }

    # Remove interfaces not being collected and sort according to interface id
    # Add more intelligent sorting later (see nmiscgi.pl)
    my @interfaces = ();
    foreach my $i ( sort { $a <=> $b } keys %interfaceTable ) {
        next if $interfaceTable{$i}->{collect} ne "true";
        push @interfaces, $interfaceTable{$i};
    }
    unless (@interfaces) {
        $c->stash( error => $c->loc( "No interfaces being collected on [_1]", $host->name ) );
        $c->detach("/soft_error");
    }
    $c->stash( interfaces => \@interfaces );
}

sub interfaces_by_id_end : PathPart('interfaces') Chained('by_id') {
    shift->do_interfaces(@_);
}

sub interfaces_by_name_end : PathPart('interfaces') Chained('by_name') {
    shift->do_interfaces(@_);
}

1;
