#!/usr/bin/perl
# Test hostgroup sync where substantial changes are made

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/lib";

use lib "/usr/local/nagios/perl/lib", "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Opsview::TestUtils;
use Test::WWW::Mechanize;

use Opsview::Schema;

use Data::Dump qw(dump);
use JSON;

my $opsview = Opsview::Schema->my_connect;
my $rs = $opsview->resultset("Hostgroups");

my $res_data;
my $content;
my $expected;

my $url_prefix="http://localhost:3000";
my $ua = Test::WWW::Mechanize->new( stack_depth => 0 );
rest_log_in( $ua, "admin", "initial", $url_prefix );
$ua->add_header( 'content-type' => "application/json" );

open F, "$Bin/var/hostgroup-rest-import.json" or die "Cannot open";
my $hostgroup_json_data;
{ 
    $/=undef;
    $hostgroup_json_data = <F>;
}
close F;

my $json = JSON->new;
my $res_json;

is( $rs->find(1)->name, "Opsview", "Got HH1 as opsview" );


$ua->put("$url_prefix/rest/config/hostgroup", Content => $hostgroup_json_data);
is( $ua->status, 200, "Got success for syncing host groups on test db") || diag( $ua->content );
$res_json = $json->decode( $ua->content );
$expected = {
    objects_updated => 25,
};
is_deeply( $res_json, $expected, "Got okay response" );

is( $rs->find(1)->name, "Opsera", "HH1 changed to Opsera" );

is( $rs->search( { matpath => "" } )->count, 0, "All matpath setup correctly" );

$ua->put("$url_prefix/rest/config/hostgroup", Content => $hostgroup_json_data);
is( $ua->status, 200, "Got success for 2nd syncing of host groups") || diag( $ua->content );
$res_json = $json->decode( $ua->content );
$expected = {
    objects_updated => 25,
};
is_deeply( $res_json, $expected, "Got okay response" );
is( $rs->find(1)->name, "Opsera", "HH1 still Opsera" );

is( $rs->search( { matpath => "" } )->count, 0, "Still all matpath setup correctly" );

is( system("/usr/local/nagios/bin/db_opsview -t db_install"), 0, "Fresh instance of Opsview" );

is( $rs->find(1)->name, "Opsview", "HH1 changed to default Opsview" );

rest_log_in( $ua, "admin", "initial", $url_prefix );
$ua->add_header( 'content-type' => "application/json" );
$ua->put("$url_prefix/rest/config/hostgroup", Content => $hostgroup_json_data);
is( $ua->status, 200, "Got success for syncing host groups on fresh system") || diag( $ua->content );
$res_json = $json->decode( $ua->content );
$expected = {
    objects_updated => 25,
};
is_deeply( $res_json, $expected, "Got okay response" );

is( $rs->find(1)->name, "Opsera", "HH1 changed to Opsera" );
is( $rs->search( { matpath => "" } )->count, 0, "Still all matpath setup correctly" );


open F, "$Bin/var/hostgroup-rest-import-duplicated.json" or die "Cannot open";
{ 
    $/=undef;
    $hostgroup_json_data = <F>;
}
close F;

$ua->put("$url_prefix/rest/config/hostgroup", Content => $hostgroup_json_data);
is( $ua->status, 200, "Got success with a complicated host group with duplicate names") || diag( $ua->content );
$res_json = $json->decode( $ua->content );
$expected = {
    objects_updated => 31,
};
is_deeply( $res_json, $expected, "Got okay response" );

is( $rs->find(1)->name, "Testing", "HH1 changed to Testing" );
is( $rs->search( { matpath => "" } )->count, 0, "Still all matpath setup correctly" );


$ua->put("$url_prefix/rest/config/hostgroup", Content => $hostgroup_json_data);
is( $ua->status, 200, "Confirm 2nd sync okay") || diag( $ua->content );
$res_json = $json->decode( $ua->content );
$expected = {
    objects_updated => 31,
};
is_deeply( $res_json, $expected, "Got okay response" );

is( $rs->search( { matpath => "" } )->count, 0, "All matpath setup correctly" );

$ua->get_ok("$url_prefix/rest/config/hostgroup?cols=-hosts,-children,-is_leaf,-id,-uncommitted&order=dependency&rows=all");
$res_json = $json->decode( $ua->content );
Opsview::Utils->remove_keys_from_hash( $res_json, ["ref"] );
$expected = $json->decode( $hostgroup_json_data );
my @received = @{$res_json->{list}};

# Remove a buildbot entry here, which was missed in the input
splice @received, 3, 1;
Opsview::Utils->remove_keys_from_hash( $expected, ["ref"] );
my @expected = @{$expected->{list}};
is_deeply( \@received, \@expected, "Data list is exactly the same as originally pushed in") || diag explain $res_json->{list};
