#!/usr/bin/perl
# Test events

use warnings;
use strict;
use FindBin qw($Bin);

use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

# Cannot add this lib before T::W::M::C
use lib "$Bin/lib";
use Opsview::TestUtils;
use Test::JSON::File;

my $ua = Test::WWW::Mechanize::Catalyst->new;
log_in( $ua, 'admin', 'initial' );

$ua->get_ok('/search/performancemetric?host=opsview&service=Opsview%20HTTP&metric=size');
like( $ua->content, qr/No type defined/ );

$ua->get_ok('/search/performancemetric?host=opsview&service=Opsview%20HTTP&metric=size&type=host');
is( $ua->content, "<ul><li>opsview</li></ul>\n" );

$ua->get_ok('/search/performancemetric?host=opsview&service=Opsview%20HTTP&metric=size&type=host&output=json');
is( $ua->content, '{"ResultSet":{"count":1,"list":["opsview"]}}' );

$ua->get_ok('/search/performancemetric?service=HTTP&metric=size&type=host');
is( $ua->content, "<ul><li>docs.opsview.org</li><li>opslave</li><li>vmhost2</li><li>www.opsview.org</li></ul>\n" );

$ua->get_ok('/search/performancemetric?host=%ops%&service=HTTP&metric=size&type=host');
is( $ua->content, "<ul><li>docs.opsview.org</li><li>opslave</li><li>www.opsview.org</li></ul>\n" );

$ua->get_ok('/search/performancemetric?host=opsview&metric=time&type=service');
is( $ua->content, "<ul><li>Nagiosplug snapshot</li><li>Opsview Application Server</li><li>Opsview Daemon</li><li>Opsview HTTP</li><li>Opsview mailing list users</li><li>SSH</li><li>Tinderbox status</li></ul>\n" );

# Need to logout - can't have a 2nd instance
$ua->get_ok("/logout");

log_in( $ua, 'somehosts', 'somehosts' );

$ua->get_ok('/search/performancemetric?host=%ops%&service=HTTP&metric=size&type=host');
is( $ua->content, "<ul><li>opslave</li></ul>\n", "somehosts can only search their specific hosts" );
