#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Web::Test;

use Opsview;
use Opsview::Contact;
use Opsview::Config;

use Test::More "no_plan";
use Test::Deep;

# Set hostname
my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# This test is to prove that when a new user logins for the first time the
# welcome page is displayer
my $ua = WWW::Mechanize->new;
use Opsview::Schema;
my $schema = Opsview::Schema->my_connect;
$schema->resultset("Contacts")->update( { show_welcome_page => 1 } );

my $res = $ua->get("http://$hostname/user/preference");
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );

$ua->field( 'login_username', 'somehosts' );
$ua->field( 'login_password', 'somehosts' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

like( $ua->content, "/welcome/i", "Found a welcome page" );

ok( !$ua->find_link( id => 'systemprefslink' ), "Non-admin welcome page doesn't link to system preferences." );

$ua->form_name("welcome_form");
$ua->current_form->action;
$ua->submit;

ok( $ua->find_link( url_regex => qr^/user/preference^ ), "Found the desired user preferences page, after the welcome." );

# This test checks to see that the welcome page isnt displayed the second time the user accesses
# the system,
$ua = WWW::Mechanize->new;

$res = $ua->get("http://$hostname/user/preference");
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );

$ua->field( 'login_username', 'somehosts' );
$ua->field( 'login_password', 'somehosts' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

ok( $ua->find_link( url_regex => qr^/user/preference^ ), "Found the desired user preferences page, after the welcome." );

# This test checks to see whether the system preference page is returned after a failed login and
# then a welcome page and continue.
$ua  = WWW::Mechanize->new;
$res = $ua->get("http://$hostname/admin/systempreference");
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );

$ua->field( 'login_username', 'dud' );
$ua->field( 'login_password', 'dud' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );

$ua->field( 'login_username', 'admin' );
$ua->field( 'login_password', 'initial' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

like( $ua->content, "/welcome/i", "Found a welcome page" );

$ua->form_name("welcome_form");
$ua->current_form->action;
$ua->submit;

TODO: {
    local $TODO = "Bug: OPS- Login page isnt retaining original destination url after a failed login.";
    like( $ua->content, "/System Preferences/i", "Found the desired system preferences page, after a failed login and welcome." );
}
