#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use LWP::UserAgent;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Web::Test;

use Opsview;
use Opsview::Contact;
use Opsview::Config;

use Test::More tests => 4;
use Test::Deep;

# Set hostname
my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# Choose User-Agent value
my $user_agent = "OpsviewTest/806";

# This test is to prove that when a user logs in, the last_update value for
# an existing user agent is updated to the current time.
use Opsview::Schema;
my $schema = Opsview::Schema->my_connect;

# Add the user agent to the table with an expired time (> 1 month old)
$schema->resultset("Useragents")->update_or_create(
    {   id          => $user_agent,
        last_update => DateTime->now()->subtract( months => 2 )
    }
);

# Check the value in the database
my $rec         = $schema->resultset("Useragents")->find($user_agent);
my $last_update = $rec->last_update;
ok( DateTime->compare( DateTime->now()->subtract( months => 1 ), $last_update ) > 0, "Last_update col currently > 1 month old: $last_update" );

# Login
my $ua = LWP::UserAgent->new;
$ua->agent($user_agent);
my %login_form = (
    login_username => 'somehosts',
    login_password => 'somehosts'
);
my $response = $ua->post( "http://$hostname/login", \%login_form );

# Did login succeed? (200 ok, or 302 redirect)
ok( $response->is_success || $response->is_redirect, "Login request succeeded: " . $response->code );
unlike( $response->content, "/login_form/i", "Login page not returned, so login was successful." );

# Check the value in the database
$rec         = $schema->resultset("Useragents")->find($user_agent);
$last_update = $rec->last_update;
ok( DateTime->compare( DateTime->now()->subtract( minutes => 1 ), $last_update ) < 0, "Last_update col was updated by login: $last_update" );
