#!perl
# Tests public URLs

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";

use Test::WWW::Mechanize;

use Test::More qw(no_plan);

use Test::JSON;

use lib "$Bin/lib";
use Opsview::TestUtils;

my $user = Test::WWW::Mechanize->new();

log_in( $user, "somehosts", "somehosts", "http://localhost:3000" );

$user->get_ok("http://localhost:3000/public/normalise_duration?q=3m+20s");
is_json( $user->content, '{"ResultSet":{"seconds":200,"normalised_duration":"3m 20s"}}' );

$user->get_ok("http://localhost:3000/public/normalise_duration?q=5h+30m");
is_json( $user->content, '{"ResultSet":{"seconds":19800,"normalised_duration":"5h 30m"}}' );

$user->get_ok("http://localhost:3000/public/normalise_duration");
is_json( $user->content, '{"ResultSet":{"seconds":0,"normalised_duration":""}}' );

$user->get_ok("http://localhost:3000/public/normalise_duration?q=1h&q=34s");
is_json( $user->content, '{"ResultSet":{"seconds":3600,"normalised_duration":"1h"}}', "Only uses first parameter" );

$user->get_ok("http://localhost:3000/public/normalise_duration?q=gibberish");
is_json( $user->content, '{"ResultSet":{"error":"gibberish not wellformed. <duration><wdhms> at /usr/local/nagios/lib/Opsview/Utils/Time.pm line 33\n"}}', "Some gibberish" );

$user->get_ok("/public/check_downtime_start_end_times?starttime=2038-03-04%2001:00:00&endtime=%2B1d%203h&comment=");
is_json( $user->content, '{"ResultSet":{"endtime":"2038-03-05 04:00:00","starttime":"2038-03-04 01:00:00"}}', "Good response from downtime data" );

$user->get("/public/check_downtime_start_end_times?starttime=8-03-04%2001:00:00&endtime=%2B1d%203h&comment=");
is( $user->status, 400, "Got correct bad response");
is_json( $user->content, '{}', "Bad response from downtime data" );


