#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More;
use Test::Deep;

plan 'no_plan';

use Opsview::Test;
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Runtime;
use Runtime::Host;
use Runtime::Service;
use Runtime::Hostgroup;
use Opsview;
use Opsview::Contact;

use lib "$Bin/lib";
use Opsview::TestUtils;

use Log::Log4perl::Level;

my $admin     = Opsview::Contact->search( { name => "admin" } )->first;
my $non_admin = Opsview::Contact->search( { name => "nonadmin" } )->first;
my $somehosts = Opsview::Contact->search( { name => "somehosts" } )->first;
my $readonly  = Opsview::Contact->search( { name => "readonly" } )->first;

isa_ok( $admin,     "Opsview::Contact", "admin contact found" );
isa_ok( $non_admin, "Opsview::Contact", "non_admin contact found" );
isa_ok( $somehosts, "Opsview::Contact", "somehosts contact found" );
isa_ok( $readonly,  "Opsview::Contact", "readonly contact found" );

my $mech = Test::WWW::Mechanize::Catalyst->new;
my $id;

# reduce unnecessary output by amending rootlogger to suppress INFO and DEBUG messages
Log::Log4perl->get_logger("")->level($WARN);

my $accessible_hostgroup = Runtime::Hostgroup->search( { name => 'Leaf2', } )->first;
my $restricted_hostgroup = Runtime::Hostgroup->search( { name => 'Leaf', } )->first;

witter('Testing on a restricted hostgroup');

# NOTE: only admin user has access to downtime hostgroups currently

log_in( $mech, 'admin', 'initial' );
witter('Checking hostgroup menus for required items');
$id = $restricted_hostgroup->id;
$mech->get_ok( 'http://localhost/state/hostgroup/' . $restricted_hostgroup->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/hostgroup/' . $restricted_hostgroup->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
{
    local $TODO = "somehosts shouldnt have access to hostgroup 3 stuff really";
    $mech->get_ok( 'http://localhost/state/hostgroup/' . $restricted_hostgroup->id . '/menus' );
    $mech->content_like(qr/View downtimes/);
    $mech->content_unlike(qr/Schedule downtime/);
    $mech->content_unlike(qr/Service detail/);
    $mech->content_like(qr/Comments/);
    $mech->content_like(qr/\/info\/hostgroup\/$id/);
}
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/hostgroup/' . $restricted_hostgroup->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

witter('Testing on an accessible hostgroup');

log_in( $mech, 'admin', 'initial' );
$id = $accessible_hostgroup->id;
$mech->get_ok( 'http://localhost/state/hostgroup/' . $accessible_hostgroup->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/hostgroup/' . $accessible_hostgroup->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
$mech->get_ok( 'http://localhost/state/hostgroup/' . $accessible_hostgroup->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/hostgroup/' . $accessible_hostgroup->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/hostgroup\/$id/);
log_out($mech);

my $accessible_host = Runtime::Host->search( { name => 'monitored_by_slave' } )->first;
my $restricted_host = Runtime::Host->search( { name => 'cisco' } )->first;

witter('Testing on a restricted host');

log_in( $mech, 'admin', 'initial' );
witter('Checking host menus for required items');
$id = $restricted_host->opsview_host_id;
$mech->get_ok( 'http://localhost/state/host/' . $restricted_host->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/View host detail/);
$mech->content_unlike(qr/Service summary/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/host\/$id/);
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_like(qr/Edit host configuration/);

$mech->get_ok( 'http://localhost/state/host/' . $restricted_host->id . '/menus?via=event' );
$mech->content_like(qr/Service summary/);

# Test that a host without an associated configuration host does not show the Edit host configuration link
$mech->get_ok("http://localhost/state/host/998/menus");
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_like(qr/Host does not exist in configuration/);

log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/host/' . $restricted_host->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/host\/$id/);
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
$mech->get_ok( 'http://localhost/state/host/' . $restricted_host->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/host\/$id/);
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/host/' . $restricted_host->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_like(qr/\/info\/host\/$id/);
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

$id=undef;

witter('Testing on an accessible host');

log_in( $mech, 'admin', 'initial' );
$mech->get_ok( 'http://localhost/state/host/' . $accessible_host->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_contains('/info/host/4', "host=4 because of opsview object");
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_like(qr/Edit host configuration/);
log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/host/' . $accessible_host->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_contains('/info/host/4');
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
$mech->get_ok( 'http://localhost/state/host/' . $accessible_host->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_contains('/info/host/4');
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/host/' . $accessible_host->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Telnet/);
$mech->content_like(qr/SSH/);
$mech->content_like(qr/Comments/);
$mech->content_contains('/info/host/4');
$mech->content_like(qr/Result:/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
log_out($mech);

my $accessible_service = Runtime::Service->search( { hostname => 'monitored_by_slave', servicename => 'VNC' } )->first;
my $restricted_service = Runtime::Service->search( { hostname => 'cisco', servicename => 'Coldstart' } )->first;

witter('Testing on a restricted service');

log_in( $mech, 'admin', 'initial' );
witter('Checking service menus for required items');
$mech->get_ok( 'http://localhost/state/service/' . $restricted_service->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_like(qr/Edit host configuration/);
$mech->content_like(qr/Edit service check configuration/);
$mech->content_like(qr/Submit check result/);

# Test that a service without an associated configuration host does not show the Edit host configuration link
$mech->get_ok("http://localhost/state/service/999/menus");
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_like(qr/Host does not exist in configuration/);

log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/service/' . $restricted_service->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_like(qr/Submit check result/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
$mech->get_ok( 'http://localhost/state/service/' . $restricted_service->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_unlike(qr/Submit check result/);
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/service/' . $restricted_service->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_unlike(qr/Submit check result/);
log_out($mech);

witter('Testing on a accessible service');
log_in( $mech, 'admin', 'initial' );
$mech->get_ok( 'http://localhost/state/service/' . $accessible_service->id . '/menus' );
$mech->content_like(qr/Schedule downtime/) || warn $mech->content;
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_like(qr/Edit host configuration/);
$mech->content_like(qr/Edit service check configuration/);
$mech->content_like(qr/Submit check result/);

# Check for acknowledge links
$mech->get_ok("http://localhost/state/host/name/doesnt_exist_1/menus");
$mech->content_contains("Acknowledge", "Ack option for host");

$mech->get_ok("http://localhost/state/service/216/menus");
$mech->content_contains("Acknowledge", "Ack options for service" );

$mech->get_ok("http://localhost/state/host/name/doesnt_exist_2/menus");
$mech->content_lacks("Acknowledge", "None because already ackd");

$mech->get_ok("http://localhost/state/service/217/menus");
$mech->content_lacks("Acknowledge", "None because already ackd");

log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
$mech->get_ok( 'http://localhost/state/service/' . $accessible_service->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_like(qr/Submit check result/);
log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
$mech->get_ok( 'http://localhost/state/service/' . $accessible_service->id . '/menus' );
$mech->content_like(qr/Schedule downtime/);
$mech->content_like(qr/View downtimes/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_like(qr/Submit check result/);
log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
$mech->get_ok( 'http://localhost/state/service/' . $accessible_service->id . '/menus' );
$mech->content_unlike(qr/Schedule downtime/);
$mech->content_like(qr/View service detail/);
$mech->content_like(qr/Last check:/);
$mech->content_unlike(qr/Edit host configuration/);
$mech->content_unlike(qr/Edit service check configuration/);
$mech->content_unlike(qr/Submit check result/);
log_out($mech);




# Test multiple service checks
log_in( $mech, 'admin', 'initial' );
$mech->get_ok("/admin/servicecheck?name=Interface: something");
$mech->content_like(qr/Edit: Interface/);
$mech->get_ok("/admin/servicecheck?name=Slave-node: another");
$mech->content_like(qr/Edit: Slave-node/);
$mech->get_ok("/admin/servicecheck?name=Cluster-node: another");
$mech->content_like(qr/Edit: Cluster-node/);

$mech->get_ok("/admin/host/menu/7", "cisco admin menu");
$mech->content_unlike(qr/No status yet - must reload/, "Should be visible in runtime" );

$mech->get_ok("/admin/host/menu/22", "monitored_by_cluster admin menu" );
$mech->content_like(qr/No status yet - must reload/, "No runtime status yet" );

$mech->get_ok("/admin/servicecheck/menu/85", "servicecheck menu" );
$mech->content_like(qr/Exceptions:/);
$mech->content_like(qr/Status/);


# Test for comments lit up
$mech->get_ok("/state/host/name/cisco3/menus");
$mech->content_contains("/images/comments-set.png", "Bulb lit for cisco3" );

$mech->get_ok("/state/host/name/cisco/menus");
$mech->content_contains("/images/comments-none.png", "Bulb not lit for cisco" );

$mech->get_ok("/state/hostgroup/9/menus");
$mech->content_contains("/images/comments-set.png", "Bulb lit for host group 9");

$mech->get_ok("/state/hostgroup/2/menus");
$mech->content_contains("/images/comments-none.png", "Bulb not lit for host group 2");

$mech->get_ok("/admin/hostgroup/menu/9");
$mech->content_contains("/images/comments-set.png", "Bulb lit for host group 9 via admin menu");

$mech->get_ok("/admin/hostgroup/menu/2");
$mech->content_contains("/images/comments-none.png", "Bulb not lit for host group 2 via admin menu");

