#!/usr/bin/perl
# Test creation of monitor packs

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/../etc";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";
use Test::More qw(no_plan);
use Opsview::Test qw(opsview);
use File::Path;
use Cwd;

my $tempdir = "/tmp/monitorpacks.$$";
my $cwd = getcwd();

sub create_temp_dir {
    chdir "$Bin";
    rmtree $tempdir;
    mkdir $tempdir or die "Cannot create $tempdir";
    chdir $tempdir or die "Cannot chdir $tempdir";
}

$ENV{OPSVIEW_URL_PREFIX} = "http://localhost:3000";

my (@full_output, $expected, @diff);

# Test creations
create_temp_dir();
@full_output = map { chop; $_ } `/usr/local/nagios/bin/create_monitor_pack --username=admin --password=initial "Monitorpack creation"`;
is( $?, 0, "Can create monitor pack files" );
$expected = [
"Created config.json",
"Created info",
"Created Makefile",
];
is_deeply( \@full_output, $expected, "Got same output" ) || diag join("\n", @full_output);

@diff = `diff --exclude=.svn -ur $Bin/var/monitorpack.expected/Attributes $tempdir`;
is( $?, 0, "No difference - as expected" ) || diag @diff;


# Test Network - Base. Also tests recreation
create_temp_dir();
@full_output = map { chop; $_ } `/usr/local/nagios/bin/create_monitor_pack --username=admin --password=initial "Network - Base"`;
is( $?, 0, "Can create monitor pack files" );
$expected = [
"Created config.json",
"Created info",
"Created Makefile",
];
is_deeply( \@full_output, $expected, "Got same output" ) || diag join("\n", @full_output);

@diff = `diff --exclude=.svn -ur $Bin/var/monitorpack.expected/Network-Base $tempdir`;
is( $?, 0, "No difference - as expected" ) || diag @diff;

# Add a plugin
open F, ">", "plugins/check_dummy_plugin" or die "Cannot create: $!";
print F <<'EOF';
#!/usr/bin/perl
print "Help text!\n";
EOF
close F;
chmod 0777, "plugins/check_dummy_plugin";

is( system("make mpack"), 0, "Can create monitor pack tar file" );

is( system("/usr/local/nagios/bin/install_monitor_pack -f application-name.tar.gz"), 0, "Can install monitor pack");

is( system("/usr/local/nagios/bin/reset_uncommitted"), 0, "Reset uncommitted flag okay");

is( system("/usr/local/nagios/bin/db_opsview -t db_backup > /usr/local/nagios/t/var/opsview.test.db.auto"), 0, "Backup of test db okay" );

is( system("diff -u /usr/local/nagios/t/var/opsview.test.db /usr/local/nagios/t/var/opsview.test.db.auto"), 0, 
    "No differences found, so create and install monitor pack process works") || diag("If correct, run: cp /usr/local/nagios/t/var/opsview.test.db.auto /usr/local/nagios/t/var/opsview.test.db");

chdir $cwd or die "Cannot chdir back to $cwd";

rmtree($tempdir);

