package Opsview::Test::WWW::Mechanize;

use warnings;
use strict;

use Test::More;

use Test::WWW::Mechanize;
use base "Test::WWW::Mechanize";

sub new {
    my ( $class, %args ) = @_;
    $args{stack_depth} = 0;
    my $opsview_url = delete $args{opsview_url} || "http://localhost:3000";
    my $user        = delete $args{username}    || "admin";
    my $pass        = delete $args{password}    || "initial";
    my $obj         = $class->SUPER::new(%args);
    $obj->{opsview_url} = $opsview_url;
    $obj->{username}    = $user;
    $obj->{password}    = $pass;
    $obj;
}

sub get_ok {
    my ( $self, @args ) = @_;
    $self->SUPER::get_ok(@args);
    my $base_url = $self->{opsview_url};
    my $errors   = 0;
    foreach my $link ( $self->links ) {
        if ( $link->url =~ /^$base_url/ ) {
            diag( "Bad link: " . $link->url );
            $errors++;
        }
    }
    $errors == 0 ? pass("All links okay") : fail("$errors link errors");
}

sub login {
    my $self = shift;
    $self->get_ok( $self->{opsview_url} );
    $self->form_name("login_form") || diag( "Can't find login_form. Page contents: " . $self->content );
    $self->field( "login_username" => $self->{username} );
    $self->field( "login_password" => $self->{password} );
    $self->field( "noscript"       => 0 );
    $self->click("login");
    $self->content_like(qr/Host Group Summary/) || diag( $self->content );
    $self;
}

1;
