#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Opsview::TestCGI;
use strict;
use base 'CGI';

use Catalyst::Test 'Opsview::Web';
use HTTP::Request;
use URI;

use Exporter;
our @ISA     = qw(Exporter);
our $VERSION = '$Revision$';
our @EXPORT  = qw(post);

sub import {
    Opsview::TestCGI->export_to_level( 1, @_ );
}

=head1 NAME

Opsview::TestCGI

=head1 SYNOPSIS

Common test functions

=head1 DESCRIPTION

Helps with test functions for Opsview

=head1 METHODS

=over 4

=item post

This is an exported function. Runs admin_apply.cgi for code not yet ported
to Catalyst (Opsview::Web), and calls the appropriate Opsview::Web URI for
code ported. Types currently ported:

  * contact
  * servicecheck
  * host

=cut

my %ported_types = map { ( $_ => 1 ) } qw/contact servicecheck host monitoringserver hosttemplate hostgroup servicegroup
    servicecheckhostexception servicecheckhosttemplateexception/;

sub post {
    my $data = join( ' ', @_ ) . ' ';
    my %params;
    while ( $data =~ m/(\w+)=(["']([^"']+?)["']|[^" ]\S*|) /g ) {
        my $key = $1;
        my $val = ( $3 ? $3 : $2 );
        push( @{ $params{$key} }, $val );
    }
    my $uri = URI->new('http://localhost/cgi-bin/admin_apply.cgi');
    if ( $ported_types{ $params{'type'}[0] } ) {
        my ( $action_name, $uri_part );
        $action_name = $params{'action'}[0];
        if ( $action_name eq 'edit' ) {
            $uri_part = 'save';
        }
        elsif ( $action_name eq 'new' ) {
            $uri_part = 'save_new';
        }
        elsif ( $action_name eq 'delete' ) {
            $uri_part = 'delete';
        }
        else {
            die "Can't translate action ${action_name}";
        }
        $uri = URI->new( join( '/', 'http://localhost/admin', $params{'type'}[0], $uri_part, ( $params{'id'}[0] ? ( $params{'id'}[0] ) : () ) ) );
    }
    $uri->query_form( \%params );
    warn "Calling $uri" if ( $ENV{VERBOSE_TESTS} );
    my $request = HTTP::Request->new( POST => $uri );
    my $response = request($request);
    return (
        ( $response->is_success && $response->content !~ m/Opsview error/ )
        ? 0
        : -1
    );
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public Licence v2

=cut

1;
